
import numpy as np
import matplotlib.pyplot as plt
from scipy.integrate import quad
import scipy
import os
import pandas as pd

filename_ref = "2022-10-22 152241 Interferograms FTIR_A_PP_24p5um_ppol_14_18mW_13ns_WL_ref_SI.txt"

wavenumber_min = 705        # 14,29 micrometer  Re(permRef)= -7673,3
wavenumber_max = 1500      #  6,67 micrometer  Re(permRef)= -2025,4
#wavenumber = np.linspace(wavenumber_min, wavenumber_max, 200)

permittivity = 11.5 # optic permitiviy of sample


# AFM
tipRadius = 25*10**(-9) #m
alpha_t = 4*np.pi*tipRadius**3
h_0 = 40*10**(-9) #m
oszFreq = 73.6*10**3 #Hz


permRef = 11.7 + 0.001*1j # permittivity of reference sample (Si)
#permRef = -5000 + 2000*1j # permittivity of reference sample (Au)
permSi  = 11.7 + 0.001*1j
harmonicFaktor = 2

# constants for reflection calculation
sin60 = np.sin(np.radians(60))
cos60 = np.cos(np.radians(60))

############## Reference calculation ########################################

### FFT parameters
dx = 800e-6                 # meter interferometer distance
sr = 1024                   # sampling rate
c  = 2.998e8                # m/s       
dt = 2*dx/(c*sr)
freq = np.fft.fftfreq(sr, dt)       # creating array of frequincies used in FFT
wavenumber = freq*33.35641/1e12     # conversion to wavenumbers

data = open(filename_ref, "r") # open data file

header = True
lineCount = 0

# loop over all lines in txt file until you find header, numberOfColumns and numberOfLines
with open(filename_ref, "r") as data:
  line = data.readline()
  while line:
    data.tell() #returns the location of the next line
    line = data.readline()

    lineCount += 1 # counting lines until data starts
    if("#" not in line):
        
        if(header == True):
            startingLine = data.tell()     # save number of starting line. 
            lineCount = 0                  # reset lineCounter
            headerArray = line.split("\t") #split data at tap
            header = False
        
               
# create a data array now that size in known
dataArray = np.zeros((len(headerArray), lineCount))    


lineCount = 0 # reset line counter
# open file again
with open(filename_ref, "r") as data:
  data.seek(startingLine-1) # jump back to line where data starts
  line = data.readline()
  while line:
    line = data.readline()
    line = line.replace("\n", "")
    linedata = line.split("\t")
    
    # fill dataarray 
    for col in range(0,len(linedata)-1,1):
        dataArray[col, lineCount] = float(linedata[col])
    
    lineCount += 1

data.close()

row   = dataArray[0]
column = dataArray[1]
run   = dataArray[2]
depth = dataArray[3]
M   = dataArray[4]
iO0A = dataArray[5]
iO0P = dataArray[6]
iO1A = dataArray[7]
iO1P = dataArray[8]
iO2A = dataArray[9]
iO2P = dataArray[10]
iO3A = dataArray[11]
iO3P = dataArray[12]
iO4A = dataArray[13]
iO4P = dataArray[14]

iO2R = np.zeros((len(iO2P))) 


# binarization of the phase and O2R calculation
for i in range(0,len(iO2P), 1):
    if (iO2P[i] < 0):
        iO2R[i] = 1*iO2A[i]
    else: 
        iO2R[i] = -1*iO2A[i]

# save all arrays in the DataFrame for pivot
dataTable_ref = pd.DataFrame({
    "row":   dataArray[0],
    "column":dataArray[1],
    "run":   dataArray[2],
    "depth": dataArray[3],
    "M":     dataArray[4],
    "iO0A":   dataArray[5],
    "iO0P":   dataArray[6],
    "iO1A":   dataArray[7],
    "iO1P":   dataArray[8],
    "iO2A":   dataArray[9],
    "iO2P":   dataArray[10],
    "iO3A":   dataArray[11],
    "iO3P":   dataArray[12],
    "iO4A":   dataArray[13],
    "iO4P":   dataArray[14],
    "iO2R":   iO2R })


# pivot data to average reference
pT_ref = pd.pivot_table(dataTable_ref, values= 'iO2R', index="row", columns= 'depth', aggfunc=np.mean) # pT is short of Pivot Table
pT_ref = pT_ref.fillna(0)      # replacing NaN values to zeros


# FFT calculation
fft_ref =  np.fft.fft(pT_ref.to_numpy())                      

# cut data to size according to given wavelength intervall
for i in range(0,sr-1, 1):
    if(wavenumber[i] < wavenumber_min): minIdx = i # find index of min Value
    if(wavenumber[i] > wavenumber_max): # find index of max value
        maxIdx = i
        break
wavenumber = wavenumber[minIdx:maxIdx]

ref = fft_ref[0,minIdx:maxIdx]

#########################################################################################################################################


def permittivityFunc(k, permittivity, omega_plasmon, damping):
    return permittivity - (omega_plasmon**2 * permittivity) / (k**2 + k*damping*1j)


def scatteringCrosssection_Ref(time):
    # calulate beta
    beta = (permRef-1)/(permRef+1)
    # calulate h
    h =tipRadius + h_0 * (1 + np.cos(oszFreq * 2 * np.pi * time))
    # reflection calulation
    tauRef  = np.arcsin(sin60 / np.sqrt(permRef))
    reflectionRef = (np.sqrt(permRef)*cos60 - np.cos(tauRef)) /\
            (np.sqrt(permRef)*cos60 + np.cos(tauRef)) 

    return ((reflectionRef + 1)**2 * alpha_t)/ \
           (1 - (beta * alpha_t)/\
                (16*np.pi*h**3))

def scatteringCrosssection(perm, time, c1, c2):
    #perm = permittivityFunc(wavenumber)
    # calulate beta
    beta = (perm-1)/(perm+1)
    # calulate h
    h =tipRadius + h_0 * (1 + np.cos(oszFreq * 2 * np.pi * time))
    # reflection calulation
    tau  = np.arcsin(sin60 / np.sqrt(perm))
    reflection = (np.sqrt(perm)*cos60 - np.cos(tau)) /\
            (np.sqrt(perm)*cos60 + np.cos(tau))
    tau0  = np.arcsin(sin60 / np.sqrt(permSi))
    reflection0 = (np.sqrt(permSi)*cos60 - np.cos(tau0)) /\
             (np.sqrt(permSi)*cos60 + np.cos(tau0)) 
    return ((reflection0*c1 + reflection*c2 + 1)**2 * alpha_t)/ \
           (1 - (beta * alpha_t)/\
                (16*np.pi*h**3))


        
def reflectionAmpl(wavenumber, omega_plasmon, damping):
    perm = permittivity - (omega_plasmon**2 * permittivity) / (wavenumber**2 + wavenumber*damping*1j)
    tau  = np.arcsin(sin60 / np.sqrt(perm))
    return np.abs((np.sqrt(perm)*cos60 - np.cos(tau)) /\
            (np.sqrt(perm)*cos60 + np.cos(tau)))
        
def reflectionPhase(wavenumber, omega_plasmon, damping):
    perm = permittivity - (omega_plasmon**2 * permittivity) / (wavenumber**2 + wavenumber*damping*1j)
    tau  = np.arcsin(sin60 / np.sqrt(perm))
    return np.angle((np.sqrt(perm)*cos60 - np.cos(tau)) /\
            (np.sqrt(perm)*cos60 + np.cos(tau)))


def scattCrossFittingFuncPhase(wavenumber, omega_plasmon, damping, c1, c2):
    
    perm = permittivity - (omega_plasmon**2 * permittivity) / (wavenumber**2 + wavenumber*damping*1j)
    
    
    real_sampleNF    = quad(lambda t: np.real(scatteringCrosssection(perm, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_sampleNF    = quad(lambda t: np.imag(scatteringCrosssection(perm, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    sampleNF         = real_sampleNF + imag_sampleNF*1j


    real_referenceNF = quad(lambda t: np.real(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_referenceNF = quad(lambda t: np.imag(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    referenceNF      = real_referenceNF+imag_referenceNF*1j
    
    
    return np.angle(sampleNF/referenceNF) + phase_shift


def scattCrossFittingFuncAmpl(wavenumber, omega_plasmon, damping, c1, c2):
    
    perm = permittivity - (omega_plasmon**2 * permittivity) / (wavenumber**2 + wavenumber*damping*1j)
    
    
    real_sampleNF    = quad(lambda t: np.real(scatteringCrosssection(perm, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_sampleNF    = quad(lambda t: np.imag(scatteringCrosssection(perm, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    sampleNF         = real_sampleNF + imag_sampleNF*1j


    real_referenceNF = quad(lambda t: np.real(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_referenceNF = quad(lambda t: np.imag(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    referenceNF      = real_referenceNF+imag_referenceNF*1j
    
    
    return np.abs(sampleNF/referenceNF)


#function for mixed resonances 
def scattCrossFittingFuncPhase2(wavenumber, omega_plasmon, damping, c1, c2, omega_plasmon2, damping2):
    
    perm = permittivity - (omega_plasmon**2 * permittivity) / (wavenumber**2 + wavenumber*damping*1j)
    perm2 = permittivity - (omega_plasmon2**2 * permittivity) / (wavenumber**2 + wavenumber*damping2*1j)
    
    
    real_sampleNF    = quad(lambda t: np.real(scatteringCrosssection(perm, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_sampleNF    = quad(lambda t: np.imag(scatteringCrosssection(perm, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    sampleNF         = real_sampleNF + imag_sampleNF*1j
    
    real_sampleNF2   = quad(lambda t: np.real(scatteringCrosssection(perm2, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_sampleNF2   = quad(lambda t: np.imag(scatteringCrosssection(perm2, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    sampleNF2        = real_sampleNF2 + imag_sampleNF2*1j
    
#    sampleNFmix      = (sampleNF*5 + sampleNF2)/6    #mixting of pumped and unpumped
    sampleNFmix      = sampleNF2 - sampleNF    #DD
    real_referenceNF = quad(lambda t: np.real(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_referenceNF = quad(lambda t: np.imag(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    referenceNF      = real_referenceNF+imag_referenceNF*1j
    
    
    return np.angle(sampleNFmix/referenceNF) + phase_shift

def scattCrossFittingFuncAmpl2(wavenumber, omega_plasmon, damping, c1, c2, omega_plasmon2, damping2):
    
    perm = permittivity - (omega_plasmon**2 * permittivity) / (wavenumber**2 + wavenumber*damping*1j)
    perm2 = permittivity - (omega_plasmon2**2 * permittivity) / (wavenumber**2 + wavenumber*damping2*1j)
    
    
    real_sampleNF    = quad(lambda t: np.real(scatteringCrosssection(perm, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_sampleNF    = quad(lambda t: np.imag(scatteringCrosssection(perm, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    sampleNF         = real_sampleNF + imag_sampleNF*1j
    
    real_sampleNF2   = quad(lambda t: np.real(scatteringCrosssection(perm2, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_sampleNF2   = quad(lambda t: np.imag(scatteringCrosssection(perm2, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    sampleNF2        = real_sampleNF2 + imag_sampleNF2*1j
    
#    sampleNFmix      = (sampleNF*5 + sampleNF2)/6    #mixting of pumped and unpumped
    sampleNFmix      = sampleNF2 - sampleNF    #DD


    real_referenceNF = quad(lambda t: np.real(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_referenceNF = quad(lambda t: np.imag(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    referenceNF      = real_referenceNF+imag_referenceNF*1j
    
    return np.abs(sampleNFmix/referenceNF)

def scattCrossFittingFunc2(wavenumber, omega_plasmon, damping, c1, c2, omega_plasmon2, damping2):
    
    perm = permittivity - (omega_plasmon**2 * permittivity) / (wavenumber**2 + wavenumber*damping*1j)
    perm2 = permittivity - (omega_plasmon2**2 * permittivity) / (wavenumber**2 + wavenumber*damping2*1j)
    
    
    real_sampleNF    = quad(lambda t: np.real(scatteringCrosssection(perm, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_sampleNF    = quad(lambda t: np.imag(scatteringCrosssection(perm, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    sampleNF         = real_sampleNF + imag_sampleNF*1j
    
    real_sampleNF2   = quad(lambda t: np.real(scatteringCrosssection(perm2, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_sampleNF2   = quad(lambda t: np.imag(scatteringCrosssection(perm2, t, c1, c2) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    sampleNF2        = real_sampleNF2 + imag_sampleNF2*1j
    
#    sampleNFmix      = (sampleNF*5 + sampleNF2)/6    #mixting of pumped and unpumped
    sampleNFmix      = sampleNF2 - sampleNF    #DD


    real_referenceNF = quad(lambda t: np.real(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    imag_referenceNF = quad(lambda t: np.imag(scatteringCrosssection_Ref(t) * np.exp(-2*np.pi*harmonicFaktor*oszFreq*t*1j)), 0, 1/oszFreq)[0]
    referenceNF      = real_referenceNF+imag_referenceNF*1j
    
    return sampleNFmix/referenceNF

vcurveAmpl = np.vectorize(scattCrossFittingFuncAmpl)
vcurvePhase = np.vectorize(scattCrossFittingFuncPhase)
vcurveAmpl2 = np.vectorize(scattCrossFittingFuncAmpl2)
vcurvePhase2 = np.vectorize(scattCrossFittingFuncPhase2)
vcurveR2     = np.vectorize(scattCrossFittingFunc2)


#Exp fit decay Wpl2
tau1 = 3.8
A1 = 0

#Exp fit decay dampiong2
tau2 = 3.8
A2 = 500 

# Unpumped resonance parameters
wpl1 = 1029
damping1 = 262
c1 = 1    #Non resonant (Si)
c2 = 0.1  #Drude (NW)
phase_shift = 0 #radians

# Calculation of spectra for different decays
maxIdx  = 10
square  = np.zeros(maxIdx)
T       = np.zeros(maxIdx)
wpl2    = np.zeros(maxIdx)
damping2= np.zeros(maxIdx)
f       = pd.DataFrame(np.zeros((len(wavenumber),maxIdx)))
f.index = wavenumber
f2 = f

# Plot the reference 
'''
fig = plt.figure(figsize=(10,10))
plt.title('Si Ref Amplitude', fontsize=20)
plt.xlabel("wavenumber[cm$^{-1}$]", fontsize=16)
plt.plot(wavenumber, np.abs(ref), linewidth = 7)
plt.show()
'''
#DD spectra 
    
fig = plt.figure(figsize=(10,10))
plt.title('Amplitude, wpl0: '+str(wpl1+A1)+'cm-1, Y0: '+str(damping1+A2)+'cm-1', fontsize=20)
plt.xlabel("wavenumber[cm$^{-1}$]", fontsize=16)
#plt.plot(wavenumber, vcurveAmpl(wavenumber, wpl, damping, c1, c2), linewidth = 7, label=" Unpumped")
    
for i in range(0,maxIdx, 1):
    T[i] = 2*i
    wpl2[i]     = wpl1 + A1*np.exp(-T[i]/tau1)
    damping2[i] = damping1 + A2*np.exp(-T[i]/tau2)
#    f[f.columns[i]] = np.abs(vcurveAmpl2(wavenumber, wpl1, damping1, c1, c2, wpl2[i], damping2[i])*ref)
    f[f.columns[i]] = np.abs(vcurveR2(wavenumber, wpl1, damping1, c1, c2, wpl2[i], damping2[i]))
    plt.plot(wavenumber, f[f.columns[i]], linewidth = 7)

plt.show()

#DD phase 
fig = plt.figure(figsize=(10,10))
plt.title('phase, wpl0: '+str(wpl1+A1)+'cm-1, Y0: '+str(damping1+A2)+'cm-1', fontsize=20)
plt.xlabel("wavenumber[cm$^{-1}$]", fontsize=16)
#plt.plot(wavenumber, vcurveAmpl(wavenumber, wpl, damping, c1, c2), linewidth = 7, label=" Unpumped")
    
for i in range(0,maxIdx, 1):
    T[i] = 2*i
    wpl2[i]     = wpl1 + A1*np.exp(-T[i]/tau1)
    damping2[i] = damping1 + A2*np.exp(-T[i]/tau2)
#    f[f.columns[i]] = np.abs(vcurveAmpl2(wavenumber, wpl1, damping1, c1, c2, wpl2[i], damping2[i])*ref)
    f2[f2.columns[i]] = np.angle(vcurveR2(wavenumber, wpl1, damping1, c1, c2, wpl2[i], damping2[i]))
    plt.plot(wavenumber, f2[f2.columns[i]], linewidth = 7)

plt.show()

#saving modeled data for plotting
#f.to_csv('FEL DD O2A modeling.csv', index=True)

#integrate the curves
square = np.zeros(maxIdx)
for i in range(0,maxIdx, 1):
    square[i] = scipy.integrate.simpson(f[f.columns[i]], wavenumber)
    #writing result into txt file
    with open('DD spectra integration.txt', 'a') as d:
        d.write('\n'+str(f.columns[i])+' '+str(square[i]))
